@@ -1,8 +1,8 @@
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ---------------------------------------------------------------------- copyright and license ---
 #
 #   file: lib/Dist/Zilla/Plugin/Manifes/Write.pm
 #
-#   Copyright © 2015 Van de Bugger
+#   Copyright © 2015, 2016 Van de Bugger.
 #
 #   This file is part of perl-Dist-Zilla-Plugin-Manifest-Write.
 #
@@ -17,7 +17,7 @@
 #   You should have received a copy of the GNU General Public License along with
 #   perl-Dist-Zilla-Plugin-Manifest-Write. If not, see <http://www.gnu.org/licenses/>.
 #
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ---------------------------------------------------------------------- copyright and license ---
 
 #pod =for :this This is C<Dist::Zilla::Plugin::Manifest::Write> module documentation. Read this if you are going to hack or
 #pod extend C<Manifest::Write>.
@@ -57,18 +57,23 @@ use namespace::autoclean;
 
 # PODNAME: Dist::Zilla::Plugin::Manifest::Write
 # ABSTRACT: Have annotated distribution manifest
-our $VERSION = 'v0.9.5'; # VERSION
+our $VERSION = 'v0.9.5_01'; # TRIAL VERSION
 
 with 'Dist::Zilla::Role::FileGatherer';
 with 'Dist::Zilla::Role::BeforeBuild';
+with 'Dist::Zilla::Role::BeforeArchive';
 with 'Dist::Zilla::Role::ErrorLogger';
+with 'Dist::Zilla::Role::FileFinderUser' => {
+    finder_arg_names => [ qw{ exclude_files } ],
+    default_finders  => [ ':NoFiles' ],
+};
 
 use Dist::Zilla::File::FromCode;
 use List::Util;
 use Module::Util qw{ is_valid_module_name };
 use Readonly;
+use Set::Object qw{};
 use String::RewritePrefix;
-use Try::Tiny;
 
 use Dist::Zilla::Role::File 5.023 ();       # Hint for `AutoPrereqs`.
     #   We do not consume the role but just require the specified version. Before this version
@@ -456,6 +461,24 @@ sub before_build {
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =method before_archive
+#pod
+#pod This method is called by C<Dist::Zilla> automatically before build the archive. The method prunes
+#pod files found by file finders specified in the C<exclude_files> option.
+#pod
+#pod =cut
+
+sub before_archive {
+    my ( $self ) = @_;
+    my $zilla = $self->zilla;
+    for my $file ( @{ $self->found_files } ) {
+        $zilla->prune_file( $file );
+    };
+    return;
+};
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =method gather_files
 #pod
 #pod This is the main method of the class. It adds a file with name C<< $self->manifest >> to the
@@ -473,8 +496,10 @@ sub gather_files {
         code_return_type    => 'bytes',
         code                => sub {
             my @list;
+            my $files = Set::Object->new( @{ $zilla->files } );
+            $files->remove( @{ $self->found_files } );
             #   Process all files in alphbetical order.
-            for my $file ( sort( { $a->name cmp $b->name } @{ $zilla->files } ) ) {
+            for my $file ( sort( { $a->name cmp $b->name } @{ $files } ) ) {
                 push( @list, {
                     name    => $self->_file_name( $file ),
                     comment => $self->_file_comment( $file )
@@ -828,7 +853,7 @@ __PACKAGE__->meta->make_immutable;
 #pod
 #pod =head1 COPYRIGHT AND LICENSE
 #pod
-#pod Copyright (C) 2015 Van de Bugger
+#pod Copyright (C) 2015, 2016 Van de Bugger
 #pod
 #pod License GPLv3+: The GNU General Public License version 3 or later
 #pod <http://www.gnu.org/licenses/gpl-3.0.txt>.
@@ -839,13 +864,13 @@ __PACKAGE__->meta->make_immutable;
 #pod
 #pod =cut
 
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ------------------------------------------------------------------------------------------------
 #
 #   file: doc/what.pod
 #
 #   This file is part of perl-Dist-Zilla-Plugin-Manifest-Write.
 #
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ------------------------------------------------------------------------------------------------
 
 #pod =encoding UTF-8
 #pod
@@ -853,8 +878,9 @@ __PACKAGE__->meta->make_immutable;
 #pod
 #pod C<Dist-Zilla-Plugin-Manifest-Write> (or C<Manifest::Write> for brevity) is a plugin for C<Dist::Zilla>, a replacement
 #pod for standard plugin C<Manifest>. C<Manifest::Write> writes I<annotated> distribution manifest: each
-#pod filename is followed by a comment explaining origin of the file: if it is a part of software,
-#pod meta information, or 3rd-party file.
+#pod filename is followed by a comment explaining origin of the file: if it is a part of software, meta
+#pod information, or 3rd-party file. Also it can B<I<exclude> built files from distribution>, e. g.
+#pod extra tests have to be built (to run) but need not be distributed.
 #pod
 #pod =cut
 
@@ -875,14 +901,16 @@ Dist::Zilla::Plugin::Manifest::Write - Have annotated distribution manifest
 
 =head1 VERSION
 
-Version v0.9.5, released on 2015-10-30 21:52 UTC.
+Version v0.9.5_01, released on 2016-11-26 21:21 UTC.
+This is a B<trial release>.
 
 =head1 WHAT?
 
 C<Dist-Zilla-Plugin-Manifest-Write> (or C<Manifest::Write> for brevity) is a plugin for C<Dist::Zilla>, a replacement
 for standard plugin C<Manifest>. C<Manifest::Write> writes I<annotated> distribution manifest: each
-filename is followed by a comment explaining origin of the file: if it is a part of software,
-meta information, or 3rd-party file.
+filename is followed by a comment explaining origin of the file: if it is a part of software, meta
+information, or 3rd-party file. Also it can B<I<exclude> built files from distribution>, e. g.
+extra tests have to be built (to run) but need not be distributed.
 
 This is C<Dist::Zilla::Plugin::Manifest::Write> module documentation. Read this if you are going to hack or
 extend C<Manifest::Write>.
@@ -1011,6 +1039,11 @@ C<Int>, read-only, not an init arg.
 This method is called by C<Dist::Zilla> automatically before build. The method checks validity of
 source and metainfo provider names.
 
+=head2 before_archive
+
+This method is called by C<Dist::Zilla> automatically before build the archive. The method prunes
+files found by file finders specified in the C<exclude_files> option.
+
 =head2 gather_files
 
 This is the main method of the class. It adds a file with name C<< $self->manifest >> to the
@@ -1173,7 +1206,7 @@ Van de Bugger <van.de.bugger@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2015 Van de Bugger
+Copyright (C) 2015, 2016 Van de Bugger
 
 License GPLv3+: The GNU General Public License version 3 or later
 <http://www.gnu.org/licenses/gpl-3.0.txt>.
